import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class DPin extends StatefulWidget {
  const DPin(
      {required this.number,
      this.hidden = false,
      this.fieldBorderColor = const Color(0xff9c1515),
      this.buttonText = "Verify",
      this.buttonColor = Colors.blue,
      this.buttonTextColor = Colors.white,
      this.buttonWidth = 100.0,
      this.withButton = false,
      this.underline = false,
      required this.onValueChanged});
  final int number;
  final bool hidden;
  final Color fieldBorderColor;
  final String buttonText;
  final Color buttonColor;
  final Color buttonTextColor;
  final double buttonWidth;
  final bool underline;
  final bool withButton;
  final ValueChanged<int> onValueChanged;
  @override
  DPinState createState() => DPinState();
}

class DPinState extends State<DPin> {
  @override
  void initState() {
    super.initState();
  }

  int sum = 0;

  @override
  Widget build(BuildContext context) {
    final _focusNodes = List.generate(widget.number, (index) => FocusNode());
    final _controller =
        List.generate(widget.number, (index) => TextEditingController());
    return SizedBox(
      height: 150,
      width: MediaQuery.of(context).size.width,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              for (var i = 0; i < widget.number; i++)
                Container(
                  decoration: BoxDecoration(
                      border: widget.underline
                          ? null
                          : Border.all(color: widget.fieldBorderColor),
                      borderRadius: BorderRadius.circular(5.0)),
                  height: 50,
                  width: 40,
                  child: Center(
                    child: TextField(
                      focusNode: _focusNodes[i],
                      controller: _controller[i],
                      keyboardType: TextInputType.number,
                      inputFormatters: <TextInputFormatter>[
                        FilteringTextInputFormatter.allow(RegExp(r'[0.0-9.9]')),
                      ],
                      textAlign: TextAlign.center,
                      decoration: InputDecoration(
                        counterText: "",
                        border: widget.underline ? null : InputBorder.none,
                        errorBorder: widget.underline ? null : InputBorder.none,
                        focusedBorder:
                            widget.underline ? null : InputBorder.none,
                      ),
                      maxLength: 1,
                      obscureText: widget.hidden,
                      onChanged: (text) {
                        if (i < widget.number - 1) {
                          if (text.isEmpty) {
                            _focusNodes[i - 1].requestFocus();
                          } else {
                            _focusNodes[i + 1].requestFocus();
                          }
                        } else if (i == widget.number - 1) {
                          for (var ins = 0; ins < widget.number; ins++) {
                            sum = int.parse(
                                sum.toString() + _controller[ins].text);
                          }
                          widget.withButton
                              ? null
                              : getValue(sum).then((_) => sum = 0);
                          FocusScope.of(context).requestFocus(FocusNode());
                        }
                      },
                    ),
                  ),
                ),
            ],
          ),
          const SizedBox(
            height: 30,
          ),
          widget.withButton
              ? SizedBox(
                  width: widget.buttonWidth,
                  child: ElevatedButton(
                    style:
                        ElevatedButton.styleFrom(primary: widget.buttonColor),
                    child: Text(
                      widget.buttonText,
                      style: TextStyle(color: widget.buttonTextColor),
                    ),
                    onPressed: () {
                      for (var ins = 0; ins < widget.number; ins++) {
                        sum = int.parse(sum.toString() + _controller[ins].text);
                      }
                      widget.onValueChanged.call(sum);
                      setState(() {
                        sum = 0;
                      });
                    },
                  ),
                )
              : const SizedBox()
        ],
      ),
    );
  }

  Future<void> getValue(s) async {
    widget.onValueChanged.call(s);
  }
}
